# -*- coding: utf-8 -*-

import grok

from hurry import yui
from zope.component import getMultiAdapter
from zope.location.interfaces import ISite
from waeup.browser.pages import (UniversityPage, FacultyContainerPage,
                                 DatacenterPage, FacultyPage, DepartmentPage,
                                 CoursePage, CertificatePage)
from waeup.browser.interfaces import IBreadcrumbContainer
from waeup.interfaces import (IUniversity, IWAeUPObject, IWAeUPXMLExporter,
                              IWAeUPXMLImporter, IFacultyContainer, IFaculty,
                              IDepartment, ICourse, ICertificate,
                              ICertificateCourse, IDataCenter)
from waeup.viewlets import LeftSidebar, ActionBar

grok.templatedir('templates')
grok.context(IWAeUPObject) # Make IWAeUPObject the default context

class BreadCrumbManager(grok.ViewletManager):
    grok.name('breadcrumbs')

class BreadCrumbs(grok.Viewlet):
    grok.context(IWAeUPObject)
    grok.viewletmanager(BreadCrumbManager)
    grok.order(1)

    def getEntries(self):
        result = []
        site = grok.getSite()
        context = self.context
        breadcrumbs = IBreadcrumbContainer(self.view)
        for breadcrumb in breadcrumbs:
            yield dict(
                title = breadcrumb.title,
                url = self.view.url(breadcrumb.context, breadcrumb.viewname)
                )

#
# waeup.app.University viewlets...
#
class Login(grok.Viewlet):
    """This viewlet allows to login in the sidebar.
    """
    grok.viewletmanager(LeftSidebar)
    grok.context(IUniversity)
    grok.view(UniversityPage)
    grok.order(2)

    def isAuthenticated(self):
        """Is the calling user authenticated?
        """
        return self.request.principal.id != 'zope.anybody'

class ManageLink(grok.Viewlet):
    grok.viewletmanager(LeftSidebar)
    grok.context(IUniversity)
    grok.view(UniversityPage)
    grok.order(5)
    # This link is only displayed, if the user is
    # allowed to use it!
    grok.require('waeup.manageUniversity')
    
    def render(self):
        return u'<div class="portlet"><a href="manage">Settings</a></div>'

class ManageFacultiesLink(grok.Viewlet):
    grok.viewletmanager(LeftSidebar)
    grok.context(IUniversity)
    grok.view(UniversityPage)
    grok.order(6)
    # This is so cool! This link is only displayed, when the user is
    # allowed to use it!
    grok.require('waeup.manageUniversity')
    
    def render(self):
        return u'<div class="portlet"><a href="faculties">Manage faculties</a></div>'

class ManageUsersLink(grok.Viewlet):
    grok.viewletmanager(LeftSidebar)
    grok.context(IUniversity)
    grok.view(UniversityPage)
    grok.order(6)
    grok.require('waeup.manageUsers')

    def render(self):
        return u'<div class="portlet"><a href="users">Manage users</a></div>'

class ManageDataCenter(grok.Viewlet):
    grok.viewletmanager(LeftSidebar)
    grok.context(IUniversity)
    grok.view(UniversityPage)
    grok.order(6)
    grok.require('waeup.manageUniversity')

    def render(self):
        return u'<div class="portlet"><a href="datacenter">Data Center</a></div>'

#
# Manage ("Edit settings") actions...
#    
class ManageUniversityAction(grok.Viewlet):
    grok.viewletmanager(ActionBar)
    grok.context(IUniversity)
    grok.view(UniversityPage)
    grok.require('waeup.manageUniversity')
    grok.order(2)
    grok.template('manageaction')

class GoToFacultiesAction(grok.Viewlet):
    grok.viewletmanager(ActionBar)
    grok.context(IUniversity)
    grok.view(UniversityPage)
    grok.require('waeup.manageUniversity')
    grok.order(3)
    grok.template('gotofacultiesaction')

class ManageFacultiesAction(ManageUniversityAction):
    grok.context(IFacultyContainer)
    grok.view(FacultyContainerPage)
    grok.template('manageaction')

class ManageDataCenterAction(ManageUniversityAction):
    grok.context(IDataCenter)
    grok.view(DatacenterPage)
    grok.template('manageaction')
    grok.require('waeup.manageUniversity')

class ManageFacultyAction(ManageUniversityAction):
    """ 'Edit settings' button for faculties.
    """
    grok.context(IFaculty)
    grok.view(FacultyPage)
    grok.template('manageaction')
    grok.require('waeup.manageUniversity')

class ManageDepartmentAction(ManageUniversityAction):
    """ 'Edit settings' button for departments.
    """
    grok.context(IDepartment)
    grok.view(DepartmentPage)
    grok.template('manageaction')
    grok.require('waeup.manageUniversity')

class ManageCourseAction(ManageUniversityAction):
    """ 'Edit settings' button for courses.
    """
    grok.context(ICourse)
    grok.view(CoursePage)
    grok.template('manageaction')
    grok.require('waeup.manageUniversity')

class ManageCertificateAction(ManageUniversityAction):
    """ 'Edit settings' button for certificates.
    """
    grok.context(ICertificate)
    grok.view(CertificatePage)
    grok.template('manageaction')
    grok.require('waeup.manageUniversity')

#
# Add actions...
#
class AddAction(grok.Viewlet):
    """Base class for 'adding actions'.

    AddActions are viewlets that render a button linked to an adding
    page. To create such a button in one of your pages, follow the
    following steps:

    - Create a new class derived from AddAction. In this class:
    
    - Set a grok.context() for the context type of your page. So, if
      your page sets grok.context(IMyFunnyType), do the same in the
      viewlet class.

    - Set the Page on which the button should appear using
      grok.view(). So, if your page class is MyFunnyPage, do
      grok.view(MyFunnyPage).

    - Set the template to use to 'addaction'. This means to use the
      'addaction.pt' template in the templates/ dir. Don't fiddle
      around with this template file, as it is used also by other
      viewlets.

    - If you want only certain authorized users to see the button, use
      grok.require(<Permission>). By default only people with
      waeup.manageUniversity permission are able to see your new
      button. Other users, however, will still see your page, only
      without the button rendered.

    - As one can define several buttons for a certain page, you can
      influence the order of buttons by setting grok.order(<num>). The
      lower the number, the more left the button will be displayed and
      vice versa. Default is 4.
      
    - Set the page name as ``addview_name`` attribute. This means, if
      the page, the button is linked to can be reached by calling
      http://localhost:8080/<current_location>/@@addfoo, then you should set::

        addview_name = 'addfoo'

      By default the target page name 'add' is assumed. You should
      have good reasons not to name your AddFormPage 'add'.

    - Set the button text by setting the ``addlink_text``
      attribute. If you set it to 'Add foobar', then on the page will
      appear a button with a small 'add stuff' icon and the text 'Add
      foobar'.
    """
    grok.viewletmanager(ActionBar)
    grok.view(FacultyContainerPage)
    grok.context(IFacultyContainer)
    grok.order(4)
    grok.template('addaction')
    grok.require('waeup.manageUniversity')
    grok.baseclass()

    addview_name = 'add'
    addlink_text = 'Add department'

    def getTargetURL(self):
        return '%s/@@%s' % (self.view.url(self.view.context), self.addview_name)

class AddFacultiesAction(AddAction):
    grok.context(IFacultyContainer)
    grok.view(FacultyContainerPage)
    grok.order(4)
    grok.template('addaction')
    grok.require('waeup.manageUniversity')

    addview_name = 'add'
    addlink_text = 'Add faculty'

class AddDepartmentsAction(AddAction):
    grok.context(IFaculty)
    grok.view(FacultyPage)
    grok.order(4)
    grok.template('addaction')
    grok.require('waeup.manageUniversity')

    addview_name = 'add'
    addlink_text = 'Add department'

class AddCertificateAction(AddAction):
    """Add-certificate button for department index page.
    """
    grok.context(IDepartment)
    grok.view(DepartmentPage)
    grok.order(3)
    grok.template('addaction')
    grok.require('waeup.manageUniversity')

    addview_name = 'addcertificate'
    addlink_text = 'Add certificate'

class AddCourseAction(AddAction):
    """Add-course button for department index page.
    """
    grok.context(IDepartment)
    grok.view(DepartmentPage)
    grok.order(4)
    grok.template('addaction')
    grok.require('waeup.manageUniversity')

    addview_name = 'addcourse'
    addlink_text = 'Add course'

class AddCertificateCourseAction(AddAction):
    """Add-course button for certificate index page.
    """
    grok.context(ICertificate)
    grok.view(CertificatePage)
    grok.order(4)
    grok.template('addaction')
    grok.require('waeup.manageUniversity')

    addview_name = 'addcertificatecourse'
    addlink_text = 'Add course'


#
# Primary navigation tabs (in upper left navigation bar)...
#
class PrimaryNavManager(grok.ViewletManager):
    """Viewlet manager for the primary navigation tab.
    """
    grok.name('primary_nav')

class PrimaryNavTab(grok.Viewlet):
    """Base for primary nav tabs.
    """
    grok.baseclass()
    grok.viewletmanager(PrimaryNavManager)
    grok.order(1)
    grok.require('waeup.manageUniversity')

    pnav = 0 # This is a kind of id of a tab. If some page provides
             # also a 'pnav' attribute with the same value (here: 0),
             # then the tab will be rendered as 'active' when the page
             # gets rendered.
             #
             # This way you can assign certain pages to certain
             # primary nav tabs. Each primary tab should therefore set
             # the 'pnav' attribute to a different value (or several
             # tabs might be rendered as active simultanously when the
             # page gets rendered.
    tab_title = u'Some Text'
    
    @property
    def link_target(self):
        return self.view.application_url()

    @property
    def active(self):
        view_pnav = getattr(self.view, 'pnav', 0)
        if view_pnav == self.pnav:
            return 'active'
        return ''

class HomeTab(PrimaryNavTab):
    """Home-tab in primary navigation.
    """
    grok.order(1)
    grok.require('waeup.Public')
    grok.template('primarynavtab')

    pnav = 0
    tab_title = u'Home'

class FacultiesTab(PrimaryNavTab):
    """Faculties-tab in primary navigation.
    """
    grok.order(2)
    grok.require('waeup.manageUniversity')
    grok.template('primarynavtab')

    pnav = 1
    tab_title = u'Faculties'

    @property
    def link_target(self):
        return self.view.application_url('faculties')

class SearchTab(PrimaryNavTab):
    """Search tab in primary navigation.
    """
    grok.order(3)
    grok.require('waeup.manageUniversity')
    grok.template('primarynavtab')

    pnav = 2
    tab_title = u'Search'

    @property
    def link_target(self):
        return self.view.application_url('@@search')
